package amigosJ2EE.integracion;

import java.sql.*;
import javax.sql.*;
import java.util.*;
import java.text.*;
import javax.naming.*;


import amigosJ2EE.negocio.*;
import amigosJ2EE.Constantes;

/**
 * Ejemplo de implementacion del patron Data Access Object. Proporciona operaciones simples
 * sobre la base de datos con respecto a la informacion de usuarios, abstrayendo la capa JDBC.
 * 
 * CUESTIONES:
 * - se podria mejorar mucho la eficiencia, por ejemplo mejorando el SQL.
 */
public class JDBCUsuarioDAO implements UsuarioDAO {
	
	private DataSource ds;
	 
	public JDBCUsuarioDAO() throws DataException {
		try {
			ds = getDataSource();
		}
		catch(Exception e) {
			throw new DataException("Error al acceder a la B.D.", e);
		}
	}
	
	/**
	 * Crear un nuevo usuario
	 * 
	 */
	public void crear(UsuarioTO usuario) throws DataException {
		ResultSet res;
		Connection con;
		Statement sql;
		String sqlCad;
		java.sql.Date fechaNacSQL;
		
		try {
			con = ds.getConnection();
			sql = con.createStatement();
			fechaNacSQL = new java.sql.Date(usuario.getFechaNac().getTime());
			sqlCad = "INSERT INTO usuarios VALUES ('" + usuario.getLogin() + "'" +
				",'" + usuario.getPassword() + "'"  
	            + ",'" + usuario.isVaron() + "'" + ",'" + fechaNacSQL.toString() + "'"
				+ ",'" + usuario.getLocalidad() + "'" 
				+ ",'" + usuario.getDescripcion() + "'" 
				 + ")";
			System.out.println(sqlCad);
			sql.executeUpdate(sqlCad);
		}
		catch(SQLException e) {
			throw new DataException("Error al insertar usuario", e);
		}
	}
	
	/**
	 * Buscar los datos de un usuario a partir de su login.
	 */
	public UsuarioTO buscar(String login) throws DataException {
	    ResultSet res;
	    Connection con;
	    Statement sql;
	  	
	    try {
		    con = ds.getConnection();
			sql = con.createStatement();
			res = sql.executeQuery("SELECT * from usuarios WHERE login='" + login + "'");
			if (res.next())
				return crearUsuarioTO(res);
			else
				return null;
	    }
	    catch(SQLException e) {
	    	throw new DataException("Error al acceder a la B.D.", e);
	    }
	}
	
	//TODO: utilizar un value list handler para hacer esta busqueda???
	/**
	 * Buscar todos los usuarios que cumplan las condiciones especificadas.
	 * 
	 */
	public UsuarioTO[] buscar(int edadMin, int edadMax, String localidad, int sexo) throws DataException {
		java.util.Date ahora, fechaInicio, fechaFin;
		GregorianCalendar cal;
		String fechaInicioCad, fechaFinCad;
		SimpleDateFormat df;
		ResultSet res;
		Connection con;
		String sqlCad;
		Statement sql;
		ArrayList usuarios;
     
		df = new SimpleDateFormat("yyyy-MM-dd");

		//calcular fechas m�nima y m�xima teniendo en cuenta la edad deseada
		cal = new GregorianCalendar();
		//fecha minima (edad m�xima)
		cal.setTime(new java.util.Date());
		cal.set(GregorianCalendar.YEAR, cal.get(GregorianCalendar.YEAR)-edadMax);
		fechaInicio = cal.getTime();
		//fecha m�xima (edad m�nima)
		cal.setTime(new java.util.Date());
		cal.set(GregorianCalendar.YEAR, cal.get(GregorianCalendar.YEAR)-edadMin);
		fechaFin = cal.getTime();
		//pasar las fechas a formato aaaa-mm-dd para utilizarlas en SQL
		fechaInicioCad = df.format(fechaInicio);
		fechaFinCad = df.format(fechaFin);
		sqlCad = "SELECT * from usuarios WHERE fechaNac>='" + fechaInicioCad +
			  "' AND fechaNac<= '" + fechaFinCad + "'";
		//b�squeda por localidad      
		if (localidad!=null)
		   sqlCad = sqlCad + "AND localidad='" + localidad + "'";
		//b�squeda por sexo
		if (sexo!=Constantes.SEXO_INDIFERENTE)
		   sqlCad = sqlCad + "AND varon='" + sexo + "'";
		System.out.println(sqlCad);
		try {
			con = ds.getConnection();
			sql = con.createStatement();
			res = sql.executeQuery(sqlCad);
			usuarios = new ArrayList();
			while (res.next())         
			   usuarios.add(crearUsuarioTO(res));
			}
		catch(SQLException e) {
		   throw new DataException("Problema al buscar usuario en la B.D.", e); 
		}
		return (UsuarioTO[]) usuarios.toArray(new UsuarioTO[0]);         
	}
	
	/**
	 * Metodo interno para crear un DTO (Data Transfer Object) con los datos del/los usuario/s
	 * leidos de la base de datos
	 * @param datos ResultSet que contiene los datos buscados
	 * @return
	 * @throws SQLException
	 */
	private UsuarioTO crearUsuarioTO(ResultSet datos) throws SQLException {
	     UsuarioTO usuario;

	     usuario = new UsuarioTO();
	     usuario.setLogin(datos.getString("login"));
	     usuario.setPassword(datos.getString("password"));
	     usuario.setFechaNac(datos.getDate("fechaNac"));
	     usuario.setLocalidad(datos.getString("localidad"));
	     usuario.setVaron(datos.getBoolean("varon"));
	     usuario.setDescripcion(datos.getString("descripcion"));
	     return usuario;
	}
	
	
	private DataSource getDataSource() throws DataException {
		
		Context contexto;
		
		
		Hashtable ht = new Hashtable ();
		ht.put(Context.INITIAL_CONTEXT_FACTORY,
				"weblogic.jndi.WLInitialContextFactory");
		ht.put(Context.PROVIDER_URL,
				"t3://localhost:7001");
		try {
			contexto = new InitialContext (ht);
			return (DataSource) contexto.lookup ("amigosJ2EE");
		}
		catch(Exception e) {
			throw new DataException("error al inicializar contexto JNDI", e);
		}

	}
	
}