package amigosJ2EE.negocio;

import amigosJ2EE.AmigosJ2EEException;


/**
 * Ejemplo de implementacin del patron Business Delegate.
 * 
 * CUESTIONES:
 * 
 * - en este caso no se implementa logica de negocio, sino que eso se deja a otras clases. No
 * obstante, hay gente que mete la logica de negocio en este patron, para simplificar.
 * - se utiliza una facade que implementa (o pasa la pelota a otras clases, eso no lo sabemos)
 * el API con todas las funciones de la capa de negocio. Una alternativa seria trabajar con
 * objetos del modelo del dominio (Business Objects).
 */
public class UsuarioDelegate {
	/**
	 * Un usuario registrado hace login en el sistema
	 * @param login login 
	 * @param password password
	 * @return un objeto con los datos del usuario
	 * @throws LoginErrorException si el login y/o el password es/son incorrectos
	 * @throws AmigosJ2EEException si se produce cualquier otro problema
	 */
	public UsuarioTO login(String login, String password) throws LoginErrorException, AmigosJ2EEException {
	   UsuarioServiceFacade facade;
	   
	   //pasarle la pelota a una facade, que es la que sabe como hacer login
	   facade = new UsuarioServiceFacade();
	   return facade.login(login, password);	   
	}
	
	/**
	 * Registro de un nuevo usuario
	 * @param usuario datos del usuario a registrar
	 * @throws RegistroException si ya existe un usuario con ese login
	 * @throws AmigosJ2EEException cualquier otro problema
	 */
	public void registro(UsuarioTO usuario) throws RegistroException, AmigosJ2EEException {	
		UsuarioServiceFacade facade;

		facade = new UsuarioServiceFacade();
		facade.registro(usuario);	   
	}
	
	public UsuarioTO[] buscar(int edadMin, int edadMax, String localidad, int sexo) throws AmigosJ2EEException {
		UsuarioServiceFacade facade;
		
		facade = new UsuarioServiceFacade();
		return facade.buscar(edadMin, edadMax, localidad, sexo);			
	}
}
