package amigosJ2EE.negocio;

import amigosJ2EE.*;
import amigosJ2EE.integracion.*;

/**
 * Ejemplo de implementacin del patron ServiceFacade, que ofrece una vision 
 * de la capa de negocio orientada a servicios, es decir, un API compuesto de
 * funciones a las que se puede llamar para solicitar servicios.
 * 
 * CUESTIONES:
 * 
 * - Posiblemente se podria hacer una implementacion mas eficiente con el patron
 * singleton, ya que en principio no mantiene estado y podria ser compartido por todos
 * los usuarios de la aplicacion.
 * - Implementa logica de negocio, que quiza se podria llevar a otras clases.
 */
public class UsuarioServiceFacade {
	/**
	 * Un usuario hace login en la aplicacion
	 * @param login el login
	 * @param password el password
	 * @return
	 * @throws LoginErrorException si el usuario no existe o el password no es correcto
	 */
	UsuarioTO login(String login, String password) throws LoginErrorException, AmigosJ2EEException {
		UsuarioDAO udao;
		UsuarioTO usuario;
		
		try {
			udao = new JDBCUsuarioDAO();
			usuario = udao.buscar(login);
			if (!usuario.getPassword().equals(password))
				throw new LoginErrorException("");
			else
				return usuario;
		} catch (DataException e) {
			throw new AmigosJ2EEException("Error al acceder a la base de datos");
		}
	}

	/**
	 * Registro de un nuevo usuario en el sistema
	 * @param usuario datos del nuevo usuario
	 */
	public void registro(UsuarioTO usuario) throws RegistroException, AmigosJ2EEException {
		UsuarioDAO udao;
		UsuarioTO usuarioExiste;
		
		try {
			//comprobar si existe un usuario con el mismo login
			udao = new JDBCUsuarioDAO();
			usuarioExiste = udao.buscar(usuario.getLogin());
			if (usuarioExiste!=null)
				throw new RegistroException("Ya existe un usuario con ese login");
			else {	
				udao.crear(usuario);
			}
		}
		catch(DataException de) {
			throw new AmigosJ2EEException("Error al crear usuario", de);			
		}
	}
	
	/**
	 * Busca todos los usuarios registrados que cumplen las condiciones especificadas.
	 * Si alguno de los parametros es null no se considera para la busqueda.	 * 
	 * @param edadMin edad minima
	 * @param edadMax edad maxima
	 * @param localidad localidad
	 * @param sexo sexo
	 * @return
	 */
	public UsuarioTO[] buscar(int edadMin, int edadMax, String localidad, int sexo) throws AmigosJ2EEException {
		UsuarioDAO udao;
		UsuarioTO[] lista = null;
		try {
			udao = new JDBCUsuarioDAO();
			lista = udao.buscar(edadMin, edadMax, localidad, sexo);
		} catch (DataException e) {
			throw new AmigosJ2EEException("Error al buscar usuario", e);
		}
		return lista;
	}
}
